@@ -853,7 +853,7 @@ class CircuitMovie(CircuitPlot):
                  node_quantity_logarithmic_colors=False, node_label="",
                  face_quantity_cmap=None, face_quantity_clim=None, face_quantity_alpha=1,
                  face_quantity_logarithmic_colors=False, face_label="",
-                 figsize=None, title=""):
+                 figsize=None, title="", fig=None):
 
         def handle_data(data, N):
             if data is None:
@@ -902,7 +902,8 @@ class CircuitMovie(CircuitPlot):
                          node_quantity_alpha=node_quantity_alpha, node_quantity_logarithmic_colors=node_quantity_logarithmic_colors,
                          face_quantity_cmap=face_quantity_cmap,
                          face_quantity_clim=face_quantity_clim, face_quantity_alpha=face_quantity_alpha,
-                         face_quantity_logarithmic_colors=face_quantity_logarithmic_colors, figsize=figsize, title=title)
+                         face_quantity_logarithmic_colors=face_quantity_logarithmic_colors, figsize=figsize,
+                         title=title, fig=fig)
 
         self.animate_interval = animate_interval
 
@@ -1397,7 +1398,7 @@ class TimeEvolutionMovie(CircuitMovie):
                  node_quantity_logarithmic_colors=False,
                  face_quantity_cmap=None, face_quantity_clim=None, face_quantity_alpha=1,
                  face_quantity_logarithmic_colors=False,
-                 figsize=None, title="", animate_interval=10):
+                 figsize=None, title="", animate_interval=10, fig=None):
 
         self.config = config
         self.problem_nr = problem_nr
@@ -1461,7 +1462,7 @@ class TimeEvolutionMovie(CircuitMovie):
                          face_quantity_cmap=face_quantity_cmap,
                          face_quantity_clim=face_quantity_clim, face_quantity_alpha=face_quantity_alpha,
                          face_quantity_logarithmic_colors=face_quantity_logarithmic_colors,
-                         figsize=figsize, title=title, animate_interval=animate_interval)
+                         figsize=figsize, title=title, animate_interval=animate_interval, fig=fig)
 
 
     _node_quantities = {
